/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.dao.filesystem;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import br.registro.dnsshim.common.repository.FileSystemUtil;
import br.registro.dnsshim.xfrd.domain.Slave;
import br.registro.dnsshim.xfrd.domain.SlaveGroup;
import br.registro.dnsshim.xfrd.domain.Zonename;
import br.registro.dnsshim.xfrd.repository.SlaveGroupRepository;

public class SlaveGroupDao implements SlaveGroupRepository {
	private static final Logger logger = Logger.getLogger(SlaveGroupDao.class);

	private static final String EXTENSION = ".slaves";
	private final String BASE_DIR;
	
	private static final String ZONES_PREFIX = "zones="; 

	public SlaveGroupDao() {
		StringBuilder root = new StringBuilder();
		try {
			// {DNSSHIM_HOME|user.home}/dnsshim/conf/xfrd/slaves/<groupname>.slaves
			if (System.getenv("DNSSHIM_HOME") == null) {
				root.append(System.getProperty("user.home"));
				root.append(File.separatorChar);
				root.append("dnsshim");
			} else {
				root.append(System.getenv("DNSSHIM_HOME"));
			}
			root.append(File.separatorChar);
			root.append("xfrd");
			root.append(File.separatorChar);
			root.append("conf");
			root.append(File.separatorChar);
			root.append("slaves");
			root.append(File.separatorChar);

			File directory = new File(root.toString());
			FileSystemUtil.makePath(directory);
			
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
		BASE_DIR = root.toString();
	}
	
	@Override
	public void save(SlaveGroup slaveGroup) throws IOException {
		if (logger.isDebugEnabled()) {
			logger.debug("Saving SlaveGroup...");
		}
		
		StringBuilder path = new StringBuilder(BASE_DIR);
		path.append(slaveGroup.getName());
		path.append(EXTENSION);
		
		File file = new File(path.toString());
		Map<String, Slave> slaves = slaveGroup.getSlaves();
		
		StringBuilder content = new StringBuilder();

		content.append(ZONES_PREFIX);
		for (Zonename zonename : slaveGroup.getZonenames()) {
			content.append(zonename.getName());
			content.append(';');
		}
		content.append(System.getProperty("line.separator"));

		for (Entry<String, Slave> entry : slaves.entrySet()) {
			Slave slave = entry.getValue();
			content.append(slave.getAddress().getHostAddress());
			content.append(' ');
			content.append(slave.getPort());
			content.append(System.getProperty("line.separator"));			
		}
		
		BufferedWriter bw = new BufferedWriter(new FileWriter(file));
		bw.write(content.toString());
		bw.close();
	}

	@Override
	public SlaveGroup findByName(String name) throws IOException {
		StringBuilder path = new StringBuilder(BASE_DIR);
		path.append(name);
		path.append(EXTENSION);
		
		File file = new File(path.toString());
		if (!file.exists()) {
			return null;
		}
	
		return parse(file);
	}

	@Override
	public boolean delete(String name) throws IOException {
		StringBuilder path = new StringBuilder(BASE_DIR);
		path.append(name);
		path.append(EXTENSION);
		
		File file = new File(path.toString());
		return file.delete();
	}

	@Override
	public List<SlaveGroup> findAll() throws IOException {
		List<SlaveGroup> groups = new ArrayList<SlaveGroup>();
		
		File groupPath = new File(BASE_DIR);
		File[] list = groupPath.listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				return (name.endsWith(EXTENSION));
			}
		});
		
		for (File file : list) {
			SlaveGroup slaveGroup = parse(file);
			groups.add(slaveGroup);
		}
		
		return groups;
	}
	
	private SlaveGroup parse(File file) throws FileNotFoundException, UnknownHostException {
		Scanner scanner = new Scanner(file);
		scanner.useDelimiter(System.getProperty("line.separator"));
		SlaveGroup slaveGroup = new SlaveGroup();
		String groupName = file.getName().substring(0, file.getName().length() - EXTENSION.length());
		slaveGroup.setName(groupName);
		
		String line = scanner.next();
		Scanner lineScanner = null;
		if (line.startsWith(ZONES_PREFIX)) {
			line = line.substring(ZONES_PREFIX.length());
			lineScanner = new Scanner(line).useDelimiter(";");
			while (lineScanner.hasNext()) {
				String zone = lineScanner.next();
				Zonename zonename = new Zonename(zone);
				slaveGroup.addZonename(zonename);
			}
		}
		
		while (scanner.hasNext()) {
			line = scanner.next();
			lineScanner = new Scanner(line).useDelimiter(" ");
			String address = lineScanner.next();
			int port = Integer.parseInt(lineScanner.next());
			
			Slave slave = new Slave();
			slave.setAddress(InetAddress.getByName(address));
			slave.setPort(port);
			slaveGroup.addSlave(slave);
		}
		return slaveGroup;
	}
}