/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.domain.logic;

import java.io.File;
import java.net.InetAddress;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import br.registro.dnsshim.common.repository.FileSystemUtil;
import br.registro.dnsshim.xfrd.dao.filesystem.SlaveConfigDao;
import br.registro.dnsshim.xfrd.domain.Slave;
import br.registro.dnsshim.xfrd.domain.SlaveConfig;
import br.registro.dnsshim.xfrd.domain.SlaveGroup;
import br.registro.dnsshim.xfrd.domain.TsigKeyInfo;
import br.registro.dnsshim.xfrd.domain.XfrdConfig;
import br.registro.dnsshim.xfrd.domain.Zonename;
import br.registro.dnsshim.xfrd.domain.SlaveConfig.ConfigOperation;
import br.registro.dnsshim.xfrd.repository.SlaveConfigRepository;

public class SlaveGroupDumper implements Runnable {
	private static final Logger logger = Logger.getLogger(SlaveGroupDumper.class);
	private final SlaveConfigRepository repository = new SlaveConfigDao();
	
	@Override
	public void run() {
		try {
			if (logger.isInfoEnabled()) {
				logger.info("Starting SlaveGroup dumper");
			}
			
			SlaveGroupCache slaveGroupCache = SlaveGroupCache.getInstance();
			XfrdConfig xfrdConfig = XfrdConfigManager.getInstance();
			
			InetAddress localhost = InetAddress.getLocalHost();
			if (localhost.isLoopbackAddress()) {
				logger.warn("Could not determine local IP address. Using loopback");
			}

			for (Entry<String, SlaveGroup> slaveGroupEntry : slaveGroupCache.entrySet()) {
				SlaveGroup slaveGroup = slaveGroupEntry.getValue();
				StringBuilder output = new StringBuilder();
				
				for (Zonename zonename : slaveGroup.getZonenames()) {
					output.append("zone \"");
					output.append(zonename.getName());
					output.append("\" { type slave; masters { ");
					output.append(localhost.getHostAddress());
					output.append(" port ");
					int port = xfrdConfig.getDnsServerConfig().getPort();
					output.append(String.valueOf(port));
					output.append("; }; file \"dnsshim");
					output.append(File.separatorChar);
					String path = FileSystemUtil.hash(zonename.getName());
					output.append(path);
					output.append(zonename.getName());
					output.append("\"; };");
					output.append(System.getProperty("line.separator"));
				}
				
				if (output.length() > 0) {
					if (logger.isDebugEnabled()){
						logger.debug("Saving zones to config file");
					}	
					SlaveConfig slaveConfig = new SlaveConfig();
					slaveConfig.setOutput(output.toString());
					slaveConfig.setGroupName(slaveGroup.getName());
					slaveConfig.setConfigOperation(ConfigOperation.CURRENT);
					repository.save(slaveConfig);
				}
				
				output = new StringBuilder();
				for (Zonename zonename : slaveGroup.getRemovedZonenames()) {
					output.append(zonename.getName());
					output.append(System.getProperty("line.separator"));
				}

				if (logger.isDebugEnabled()) {
					logger.debug("Saving removed zones to config file");
				}
				SlaveConfig slaveConfig = new SlaveConfig();
				slaveConfig.setOutput(output.toString());
				slaveConfig.setGroupName(slaveGroup.getName());
				slaveConfig.setConfigOperation(ConfigOperation.REMOVED);
				repository.save(slaveConfig);

				slaveGroup.clearRemovedZonenames();
			}

			String osName = System.getProperty("os.name").toLowerCase();
			
			if (osName.contains("windows")) {
				logger.warn("Cannot synchronize slaves using Windows platform.");
			} else if (!xfrdConfig.isSlaveSyncEnabled()) {
				if (logger.isDebugEnabled()) {
					logger.debug("DNSSHIM will not synchronize slaves.");
				}
			} else {
				if (logger.isInfoEnabled()) {
					logger.info("Starting slaves synchronization.");
				}
				String path = repository.getPath();
				TsigKeyInfoCache tsigCache = TsigKeyInfoCache.getInstance();
				for (Entry<String, SlaveGroup> slaveGroupEntry : slaveGroupCache.entrySet()) {
					SlaveGroup slaveGroup = slaveGroupEntry.getValue();
					for (Entry<String, Slave> slaveEntry : slaveGroup.getSlaves().entrySet()) {
						Slave slave = slaveEntry.getValue();
						String hostAddress = slave.getAddress().getHostAddress();
						StringBuilder command = new StringBuilder();
						TsigKeyInfo tsig = tsigCache.get(hostAddress);
						if (tsig == null || tsig.isEmpty()) {
							logger.warn("Server " + hostAddress + " has no TSIG KEY.");
							continue;
						}
						
						Map<String, String> keys = tsig.getKeys();
						Entry<String, String> key = keys.entrySet().iterator().next();
						command.append(xfrdConfig.getSlaveSyncPath());
						command.append("SlaveSync.sh");
						command.append(' ');
						command.append(path);
						command.append(' ');
						command.append(slaveGroup.getName());
						command.append(' ');
						command.append(xfrdConfig.getSlaveUser());
						command.append(' ');
						command.append(slave.getAddress().getHostAddress());
						command.append(' ');
						command.append(xfrdConfig.getSlavePath());
						command.append(' ');
						command.append(key.getKey());
						command.append(' ');
						command.append(key.getValue());
						
						if (logger.isDebugEnabled()) {
							logger.debug("Executing rsync: " + command);
						}
						
						Runtime.getRuntime().exec(command.toString());
					}
				}
			}
				
			if (logger.isInfoEnabled()) {
				logger.info("SlaveGroup dump finished");
			}
			
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		
	}

}
